" ===============================================================
"  News Checker Parse
"  Kyosuke Takayama (http://espion.just-size.jp/)
"
"  $Id: news-checker-phead.vim,v 1.8 2004/06/10 01:34:15 gaba Exp $
" ===============================================================

scriptencoding euc-jp

if exists('g:newschecker_disable')
   finish
endif

" -------------------
" BASE
" -------------------

"" BASE --------------------------------------------------------------
function! NewsCheckerParseArticle_BASE(result)
   let result = a:result

   while result != ''
      let idx = match(result, "\<NL>")
      let buf = strpart(result, 0, idx)
      let result = strpart(result, idx+1)
      call append('$', buf)
   endwhile
endfun

"" START to END ------------------------------------------------------
function! NewsCheckerParseArticle_ste(url, enc, start, end)
   let result = system(g:newschecker_curlprg.' -s "'.a:url.'"')."\<NL>"
   if a:enc != &enc
      let result = iconv(result, a:enc, &enc)
   endif
   let result = substitute(result, '<p>', "<p>\<NL>", 'gi')
   let result = substitute(result, '<br>', "<br>\<NL>", 'gi')
   let result = substitute(result, '<h\(.\)>', '<h\1>'."\<NL>", 'gi')
   let result = substitute(result, '</h\(.\)>', '</h\1>'."\<NL>", 'gi')
   let result = substitute(result, '</div>', "\<NL></div>", 'gi')

   while result != ''
      let idx = match(result, "\<NL>")
      let buf = strpart(result, 0, idx)
      let result = strpart(result, idx+1)
      if buf =~ a:start
         let delete_flag = 0
         while result != ''
            let idx = match(result, "\<NL>")
            let buf = strpart(result, 0, idx)
            let result = strpart(result, idx+1)
            if buf =~ a:end
               break
            endif
            if g:newschecker_w3mprg == ''
               let buf = substitute(buf, '<.\{-\}>', '', 'g')
               let buf = substitute(buf, '\v^\s+', '', '')
               if delete_flag == 1
                  if buf =~ '>'
                     let buf = substitute(buf, '.*>', '', 'g')
                     let delete_flag = 0
                  else
                     let buf = ''
                  endif
               elseif buf =~ '<'
                  let delete_flag = 1
                  let buf = substitute(buf, '<.*', '', 'g')
               endif
               let buf = NewsCheckerDecode(buf)
            endif

            call append('$', buf)
            if g:newschecker_w3mprg == ''
               while strlen(buf) > g:newschecker_textwidth
                  call cursor('$', g:newschecker_textwidth)
                  exe "normal! i\<CR>"
                  let buf = getline('.')
               endwhile
            endif
         endwhile
         break
      endif
   endwhile
endfun

"" www ---------------------------------------------------------------
function! NewsCheckerParseArticle_www(url)
endfun

function! NewsCheckerParseLink_www(link)
   return a:link
endfun

function! NewsCheckerParseDate_www(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_www(url)
endfun

function! NewsCheckerParseMode_www(mode)
   return 0
endfun

" -------------------
" News Site
" -------------------

"" OSDL (Japanlinux.com) ---------------------------------------------
function! NewsCheckerParseArticle_jlc(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', '<', '</html>')
endfun

function! NewsCheckerParseLink_jlc(link)
   let link = a:link
   let link = substitute(link, '\v^http://japan.linux.com/[^/]+/(.{-}).shtml.*', '\1', '')
   let link = 'http://japan.linux.com/print.pl?sid='.link

   return link
endfun

function! NewsCheckerParseDate_jlc(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_jlc(url)
endfun

function! NewsCheckerParseMode_jlc(mode)
   if a:mode == 0
      return 1
   endif
   return 0
endfun

"" Asahi.com ---------------------------------------------------------
function! NewsCheckerParseArticle_Asahi(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', 'FJZONE START NAME="HONBUN"', 'FJZONE END NAME="HONBUN"')
endfun

function! NewsCheckerParseLink_Asahi(link)
   return a:link
endfun

function! NewsCheckerParseDate_Asahi(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_Asahi(url)
endfun

function! NewsCheckerParseMode_Asahi(mode)
   return 0
endfun

"" Mycom PC Web ------------------------------------------------------
function! NewsCheckerParseArticle_PcWatch(url)
   call NewsCheckerParseArticle_ste(a:url, 'cp932', '#BeginEditable "contents"', '#BeginEditable "RelativeArticle"')
endfun

function! NewsCheckerParseLink_PcWatch(link)
   return a:link
endfun

function! NewsCheckerParseDate_PcWatch(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_PcWatch(url)
endfun

function! NewsCheckerParseMode_PcWatch(mode)
   if a:mode == 0
      return 1
   endif
   return 0
endfun

"" CNET --------------------------------------------------------------
function! NewsCheckerParseArticle_CNET(url)
   call NewsCheckerParseArticle_ste(a:url, 'cp932', '<div class="leaf_body">', '<div class="pl10 pr15 mb10">')
endfun

function! NewsCheckerParseLink_CNET(link)
   return a:link
endfun

function! NewsCheckerParseDate_CNET(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_CNET(url)
endfun

function! NewsCheckerParseMode_CNET(mode)
   return 0
endfun

"" Yahoo! ------------------------------------------------------------
function! NewsCheckerParseArticle_YahooCom(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', '<!-- TextStart -->', '<!-- TextEnd -->')
endfun

function! NewsCheckerParseLink_YahooCom(link)
   let link = substitute(a:link, '&amp;', '\&', 'g')
   let link = substitute(link, '.*/\*\(http://.*\)', '\1', '')
   return link
endfun

function! NewsCheckerParseDate_YahooCom(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_YahooCom(url)
endfun

function! NewsCheckerParseMode_YahooCom(mode)
   if a:mode == 0
      return 1
   endif
   return 0
endfun

" -------------------
" From My RSS
" -------------------

"" YahooJapan! -------------------------------------------------------
function! NewsCheckerParseArticle_M_Yahoo(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', '<font size=5><b>', 'Υ˥塼ˤĤä')
endfun

function! NewsCheckerParseLink_M_Yahoo(link)
   return a:link
endfun

function! NewsCheckerParseDate_M_Yahoo(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_M_Yahoo(url)
   let url = substitute(a:url, '\v.*-(\d+-\w+-\w+)$', '\1', '')
   if url == a:url
      let url = ''
   endif
   return url
endfun

function! NewsCheckerParseMode_M_Yahoo(mode)
   if a:mode == 0
      return 1
   endif
   return 0
endfun

" -------------------
" From bulknews
" -------------------

"" Internet Watch (bulknews) -----------------------------------------
function! NewsCheckerParseArticle_b_InterW(url)
   call NewsCheckerParseArticle_ste(a:url, 'cp932', '<!-- ֥ȥ  -->', '<!-- ʸλ -->')
endfun

function! NewsCheckerParseLink_b_InterW(link)
   return a:link
endfun

function! NewsCheckerParseDate_b_InterW(date, link)
   let date = substitute(a:link, '\v.*/(\d{4}/\d{2}/\d{2})/(\d+).html', '\1_\2', '')
   let date = substitute(date, '/', '-', 'g')
   return date
endfun

function! NewsCheckerParseFile_b_InterW(url)
endfun

function! NewsCheckerParseMode_b_InterW(mode)
   return 0
endfun

"" K-tai Watch (bulknews) --------------------------------------------
function! NewsCheckerParseArticle_b_KtaiW(url)
   call NewsCheckerParseArticle_ste(a:url, 'cp932', '<!-- ʸ -->', '<!-- ʸλ -->')
endfun

function! NewsCheckerParseLink_b_KtaiW(link)
   return a:link
endfun

function! NewsCheckerParseDate_b_KtaiW(date, link)
   let date = substitute(a:link, '\v.*/(\d+).html', '\1', '')
   return date
endfun

function! NewsCheckerParseFile_b_KtaiW(url)
endfun

function! NewsCheckerParseMode_b_KtaiW(mode)
   return 0
endfun

"" ITmedia (bulknews) ------------------------------------------------
function! NewsCheckerParseArticle_b_ITmedia(url)
   call NewsCheckerParseArticle_ste(a:url, 'cp932', '<!--֥ȥ饯-->', '<!--ʸλ-->')
endfun

function! NewsCheckerParseLink_b_ITmedia(link)
   return a:link
endfun

function! NewsCheckerParseDate_b_ITmedia(date, link)
   let date = substitute(a:link, '\v.*/(\d+)(\d{2})/(\d+)/news(\d+).html', '\1-\2-\3_\4', '')
   return date
endfun

function! NewsCheckerParseFile_b_ITmedia(url)
endfun

function! NewsCheckerParseMode_b_ITmedia(mode)
   return 0
endfun

"" Net Security (bulknews) -------------------------------------------
function! NewsCheckerParseArticle_b_NetSec(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', '<TD valign="top" width="569">', ' contents end')
endfun

function! NewsCheckerParseLink_b_NetSec(link)
   return a:link
endfun

function! NewsCheckerParseDate_b_NetSec(date, link)
   let date = substitute(a:link, '\v.*/(\d+).html', '\1', '')
   return date
endfun

function! NewsCheckerParseFile_b_NetSec(url)
endfun

function! NewsCheckerParseMode_b_NetSec(mode)
   return 0
endfun

" -------------------
" BLOG 
" -------------------

"" Movabletype -------------------------------------------------------
function! NewsCheckerParseArticle_mova(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', ' class="title">', '<span class="posted">')
endfun

function! NewsCheckerParseLink_mova(link)
   return a:link
endfun

function! NewsCheckerParseDate_mova(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_mova(url)
endfun

function! NewsCheckerParseMode_mova(mode)
   return 0
endfun

"" Movabletype UTF-8 -------------------------------------------------
function! NewsCheckerParseArticle_movaU(url)
   call NewsCheckerParseArticle_ste(a:url, 'UTF-8', ' class="title">', '<span class="posted">')
endfun

function! NewsCheckerParseLink_movaU(link)
   return a:link
endfun

function! NewsCheckerParseDate_movaU(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_movaU(url)
endfun

function! NewsCheckerParseMode_movaU(mode)
   return 0
endfun

"" Movabletype 3 -----------------------------------------------------
function! NewsCheckerParseArticle_mova3(url)
   call NewsCheckerParseArticle_ste(a:url, 'UTF-8', ' class="content">', ' class="posted">')
endfun

function! NewsCheckerParseLink_mova3(link)
   return a:link
endfun

function! NewsCheckerParseDate_mova3(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_mova3(url)
endfun

function! NewsCheckerParseMode_mova3(mode)
   return 0
endfun

"" hatena diary ------------------------------------------------------
function! NewsCheckerParseArticle_hatena(url)
   let href = substitute(a:url, '\v.*/(\d+#.+)$', '\1', '')
   let name = substitute(a:url, '\v.*/\d+#(.+)$', '\1', '')
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', 'a href="./'.href.'" name="'.name.'"', '</div>')
endfun

function! NewsCheckerParseLink_hatena(link)
   return a:link
endfun

function! NewsCheckerParseDate_hatena(date, link)
   let date = a:date
   if a:link =~ '\v.*/\d+#p\d+$'
      let link = substitute(a:link, '\v.*/\d+#p(\d+)$', '\1', '')
      let link = strlen(link) == 1 ? '0'.link : link
      let date = substitute(a:date, '00$', link, '')
   endif
   return date
endfun

function! NewsCheckerParseFile_hatena(url)
   let anchor = substitute(a:url, '\v.*#(.{-})', '\1', '')
   return anchor
endfun

function! NewsCheckerParseMode_hatena(mode)
   return 0
endfun

"" Jugem -------------------------------------------------------------
function! NewsCheckerParseArticle_jugem(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', '<div class="entry_body">', '<div class="entry_state">')
endfun

function! NewsCheckerParseLink_jugem(link)
   return a:link
endfun

function! NewsCheckerParseDate_jugem(date, link)
   let date = a:date
   let date = substitute(date, '\v(\D)(\d)(\D)', '\1'.'0'.'\2\3', 'g')
   return date
endfun

function! NewsCheckerParseFile_jugem(url)
endfun

function! NewsCheckerParseMode_jugem(mode)
   return 0
endfun

"" cocolog -----------------------------------------------------------
function! NewsCheckerParseArticle_cocolog(url)
   call NewsCheckerParseArticle_ste(a:url, 'UTF-8', '/">ȥåץڡ</a>', '<p class="posted">')
endfun

function! NewsCheckerParseLink_cocolog(link)
   return a:link
endfun

function! NewsCheckerParseDate_cocolog(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_cocolog(url)
endfun

function! NewsCheckerParseMode_cocolog(mode)
   return 0
endfun

"" tDiary ------------------------------------------------------------
function! NewsCheckerParseArticle_tDiary(url)
   let name = substitute(a:url, '\v.*\d+[^/]*#(.+)$', '\1', '')
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', 'a name="'.name.'"', '</div>')
endfun

function! NewsCheckerParseLink_tDiary(link)
   return a:link
endfun

function! NewsCheckerParseDate_tDiary(date, link)
   let date = substitute(a:link, '\v.*(\d{4})(\d{2})(\d{2})[^/]*#.(\d+)', '\1-\2-\3_00:00:\4', '')
   if date == a:link || a:link =~ '\v#c\d+'
      let date = ''
   endif
   return date
endfun

function! NewsCheckerParseFile_tDiary(url)
   let anchor = substitute(a:url, '\v.*#(.{-})', '\1', '')
   return anchor
endfun

function! NewsCheckerParseMode_tDiary(mode)
   return 0
endfun

"" blosxom -----------------------------------------------------------
function! NewsCheckerParseArticle_blosxom(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', 'class="title">', '</div>')
endfun

function! NewsCheckerParseLink_blosxom(link)
   return a:link
endfun

function! NewsCheckerParseDate_blosxom(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_blosxom(url)
endfun

function! NewsCheckerParseMode_blosxom(mode)
   return 0
endfun

"" Livedoor ----------------------------------------------------------
function! NewsCheckerParseArticle_livedoor(url)
   call NewsCheckerParseArticle_ste(a:url, 'euc-jp', 'class="title">', '<div class="posted">')
endfun

function! NewsCheckerParseLink_livedoor(link)
   return a:link
endfun

function! NewsCheckerParseDate_livedoor(date, link)
   return a:date
endfun

function! NewsCheckerParseFile_livedoor(url)
endfun

function! NewsCheckerParseMode_livedoor(mode)
   return 0
endfun


