" ===============================================================
"  News Checker Main
"  Kyosuke Takayama (http://espion.just-size.jp/)
"
"  $Id: news-checker.vim,v 1.9 2004/06/10 01:34:15 gaba Exp $
" ===============================================================

scriptencoding euc-jp

if exists('g:newschecker_disable')
   finish
endif

"" Main --------------------------------------------------------------
function! s:NewsChecker()
   if !has('iconv')
      echohl ErrorMsg
      echo "News Checker requires Vim to have +iconv support."
      echohl None
      return
   endif

   let file = g:newschecker_file
   silent! new
   exe 'edit ++enc='.&enc.' ++ff='.&ff.' '.file
   setlocal nowrap

   command! -buffer -nargs=? NewsCheckerReadHeadline call s:NewsCheckerReadHeadline(<args>)
   command! -buffer -nargs=1 NewsCheckerReadArticle  call s:NewsCheckerReadArticle(<args>)
   command! -buffer -nargs=1 NewsCheckerScrollArticle  call s:NewsCheckerScrollArticle(<args>)
   command! -buffer -nargs=? ReadAllArticle  call s:NewsCheckerReadAllArticle()

   nnoremap <silent> <buffer> R :NewsCheckerReadHeadline<cr>
   nnoremap <silent> <buffer> <cr> :NewsCheckerReadArticle(0)<cr>
   nnoremap <silent> <buffer> W :NewsCheckerReadArticle(1)<cr>
   nnoremap <silent> <buffer> J :NewsCheckerScrollArticle(0)<cr>
   nnoremap <silent> <buffer> K :NewsCheckerScrollArticle(1)<cr>

   augroup newschecker
   autocmd! CursorHold news.newschecker call s:NewsCheckerUpdate()
   augroup END

   if has('syntax')
      highlight ShowLine cterm=underline ctermfg=blue
      syntax match Comment display ' \* '
   endif
endfun

"" Auto Update -------------------------------------------------------
function! s:NewsCheckerUpdate()
   if has('syntax')
      syntax match ShowLine /^.*\%#.*$/
   endif
   let i = 0
   while i < s:all_num
      let ltime = exists('b:newschlasttime'.i) ? b:newschlasttime{i} : 0
      if ltime + g:newschecker_num{i}_update < localtime()
         call s:NewsCheckerReadHeadline(i)
      endif
      let i = i + 1
   endwhile
endfun

"" Read Headline -----------------------------------------------------
function! s:NewsCheckerReadHeadline(...)
   echohl WarningMsg
   call s:NewsCheckerCheckUpdatetime()
   if exists('a:1')
      call s:NewsCheckerParseHeadline(a:1)
   else
      let i = 0
      while i < s:all_num
         call s:NewsCheckerParseHeadline(i)
         let i = i + 1
         silent! redraw
      endwhile
   endif
   echo 'Done...'
   echohl None

   silent! update!
endfun

"" Check Last Update -------------------------------------------------
function! s:NewsCheckerCheckUpdatetime()
   let i = 1

   while i <= line('$')
      let line = getline(i)
      if line =~ '^LastUpdate'
         let f     = substitute(line, '\v^LastUpdate(\d+): .*', '\1', '')
         let time  = substitute(line, '\v^LastUpdate\d+: (.{-}) .*', '\1', '')
         let s:last_{f} = time
      endif
      let i = i + 1
   endwhile

   let i = 0
   while  i < s:all_num
      if !exists('s:last_'.i)
         let s:last_{i} = 0
      endif
      let i = i + 1
   endwhile
endfun

"" Set Update Time ---------------------------------------------------
function! s:NewsCheckerSetUpdatetime(pattern, date)
   let i = 0
   while i <= line('$')
      let line = getline(i)
      if line =~ '^LastUpdate'.a:pattern
         call setline(i, 'LastUpdate'.a:pattern.': '.a:date.' ('.g:newschecker_num{a:pattern}_title.')')
         return
      endif
      let i = i + 1
   endwhile

   call append('$', 'LastUpdate'.a:pattern.': '.a:date.' ('.g:newschecker_num{a:pattern}_title.')')
endfun

"" Parse Headline ----------------------------------------------------
function! s:NewsCheckerParseHeadline(pattern)
   let pattern  = a:pattern
   let b:newschlasttime{pattern} = localtime()
   echo 'DownloadHeadline...'.g:newschecker_num{pattern}_title
   let result = system(g:newschecker_curlprg.' -s "'.g:newschecker_num{pattern}_url.'"') . "\<NL>"
   echo 'ParseHeadline...'.g:newschecker_num{pattern}_title
   let f = 0
   let tmp_last = 0
   let enc = 'UTF-8'
   if &ff == 'unix' && result =~ '
'
      let result = substitute(result, '
', '', 'g')
   endif

   let list_num = 0
   while result != ''
      let idx = match(result, "\<NL>")
      let buf = strpart(result, 0, idx)
      let buf = substitute(buf, '\v^\s+', '', '')
      let result = strpart(result, idx+1)
      let result = substitute(result, '\v^\s+', '', '')

      if buf =~ 'xml.* encoding="'
         let enc = substitute(buf, '\v.* encoding\="(.+)".*', '\1', '')
      endif

      if buf =~ '^<item'
         let date  = 0
         let link  = ''
         let title = ''
         let add_line = ''

         while result !~ '^</item>' && result != ''
            let idx = match(result, "\<NL>")
            let buf = strpart(result, 0, idx)
            let buf = substitute(buf, '\v^\s+', '', '')
            let result = strpart(result, idx+1)
            let result = substitute(result, '\v^\s+', '', '')

            if buf =~ '^<title'
               let buf   = iconv(buf, enc, &enc)
               let title = substitute(buf, '^<title[^>]*>\(.*\)</title>$', '\1', '')
               let title = NewsCheckerDecode(title)
            elseif buf =~ '^<link>'
               let link = substitute(buf, '^<link>\(.*\)</link>$', '\1', '')
               let link = NewsCheckerParseLink_{g:newschecker_num{pattern}_ident}(link)
               let list_num = list_num + 1
            elseif buf =~ '^<dc:date>'
               let date = substitute(buf, '^<dc:date>\(.*\)</dc:date>$', '\1', '')
            elseif buf =~ '^<pubDate>'
               let date = substitute(buf, '^<pubDate>.... \(.*\) .\{-\}</pubDate>$', '\1', '')
               let date = s:NewsCheckerParsePubDate(date)
            endif
         endwhile

         let last = NewsCheckerParseDate_{g:newschecker_num{pattern}_ident}(date, link)
         if date == 0
            let date = last
         endif

         if last > s:last_{pattern}
            let time = substitute(date, '\v(.*)\+.*', '\1', '')
            let time = substitute(time, 'T', '_', '')
            let time = substitute(time, '\v(\D)(\d)(\D)', '\1'.'0'.'\2\3', 'g')
            let title = g:newschecker_num{pattern}_title.': '.title . ' '
            let space = 140 - (strlen(title) + strlen(time))
            let i = 0
            while i < space
               let title = title.' '
               let i = i + 1
            endwhile
            let f = f + 1
            let append_list_{f} = time. ' * '.title . link . ' ' . pattern

            if last > tmp_last
               let tmp_last = last
            endif
         elseif NewsCheckerParseMode_{g:newschecker_num{pattern}_ident}(0) == 1
            break
         endif
      endif
   endwhile

   if f >= 1
      call s:NewsCheckerSetUpdatetime(pattern, tmp_last)
   endif
   while f >= 1
      call append('1', append_list_{f})
      let f = f - 1
   endwhile
endfun

"" Read Article ------------------------------------------------------
function! s:NewsCheckerReadArticle(mode)
   let line  = getline('.')
   if line !~ '^\v\d+'
      return -1
   endif
   let time   = substitute(line, '\v^(\S+) . .*', '\1', '')
   let state  = substitute(line, '\v^\S+ (.) .*', '\1', '')
   let title  = substitute(line, '\v^\S+ . (.+)\s+http.{-} \d+', '\1', '')
   let url    = substitute(line, '\v^\S+ . .+\s+(http.{-}) \d+', '\1', '')
   let num    = substitute(line, '\v^\S+ . .+\s+http.{-} (\d+)', '\1', '')
   let time   = substitute(time, ':', '-', 'g')
   let file   = g:newschecker_cachedir
   let file   = g:newschecker_comp_orememo == 1 ? file.'news.'.num.'.orememo/' : file.num.'/'
   if !isdirectory(file)
      let file = escape(file, '"')
      let res  = system('mkdir "'.file.'"')
   endif
   let file   = file.time

   let add = NewsCheckerParseFile_{g:newschecker_num{num}_ident}(url)
   if add != ''
      let file = file . '_' . add
   endif

   let flag = 0
   let file = file.g:newschecker_ext
   if state == '*'
      call cursor(line('.'), 1)
      normal! f*
      normal! r-
      silent! update!
      let flag = 1
   endif

   if g:newschecker_num{num}_ident == 'www' || a:mode == 1 || add =~ '^http://'
      let cmd = substitute(g:newschecker_wwwprg, '%URL%', url, 'g')
      let cmd = escape(cmd, '%#')
      exe '!'.cmd
   else
      let winno = bufwinnr('news.newschecker')
      silent! exe 'wincmd w'
      let winno2 = winnr()
      if winno == winno2
         exe 'new'
      endif

      echohl WarningMsg
      echo 'ReadArticle...'.title

      if (flag || getfsize(expand(file)) < 0) && g:newschecker_w3mprg != ''
         let tmp  = file
         let file = tempname()
      endif

      silent! exe 'edit ++enc='.&enc.' ++ff='.&ff.' +1 '.file
      silent! setlocal bufhidden=delete nobackup noswf nobuflisted

      if flag || getfsize(expand(file)) < 0
         if g:newschecker_w3mprg != ''
            call NewsCheckerParseArticle_{g:newschecker_num{num}_ident }(url)
            silent! update!
            let result = system(g:newschecker_w3mprg.' -dump -T "text/html" -cols '.g:newschecker_textwidth.' "'.file.'"')
            silent! exe 'edit ++enc='.&enc.' ++ff='.&ff.' +1 '.tmp
            silent! setlocal bufhidden=delete nobackup noswf nobuflisted
         endif
         normal! dG
         call append('0', '= '.title)
         call append('1', '')
         call append('2', 'Date: '.time)
         call append('3', 'URL : '.url)
         if g:newschecker_w3mprg != ''
            call NewsCheckerParseArticle_BASE(result)
         else
            call NewsCheckerParseArticle_{g:newschecker_num{num}_ident }(url)
         endif
         silent! %s/\s\+$//
         if &ff == 'unix'
            silent! %s/
//g
         endif
         silent! %s/\n\n\+/\r\r/
         normal! gg
         silent! update!
      endif

      let winno = bufwinnr('news.newschecker')
      silent! exe winno.'wincmd w'
      echo 'Done...'
      echohl None
   endif
   silent! redraw
endfun

"" Scroll Article ----------------------------------------------------
function! s:NewsCheckerScrollArticle(mode)
   let winno = winnr()
   silent! exe 'wincmd w'
   if a:mode == 1
      exe "normal! \<C-Y>"
   else
      exe "normal! \<C-E>"
   endif
   silent! exe winno.'wincmd w'
endfun

"" Read All Article --------------------------------------------------
function! s:NewsCheckerReadAllArticle()
   normal! gg
   let i = 1

   while i <= line('$')
      normal! j
      call s:NewsCheckerReadArticle(0)
      let i = i + 1
   endwhile
endfun

"" Decode ------------------------------------------------------------
function! NewsCheckerDecode(result)
   let result = a:result
   let result = substitute(result, '&#039;', "'", 'g')
   let result = substitute(result, '&gt;', '>', 'g')
   let result = substitute(result, '&lt;', '<', 'g')
   let result = substitute(result, '&quot;', '"', 'g')
   let result = substitute(result, '&apos;', "'", 'g')
   let result = substitute(result, '&nbsp;', ' ', 'g')
   let result = substitute(result, '&amp;', '\&', 'g')
   let result = substitute(result, '&raquo;', '', 'g')
   let result = substitute(result, '&laquo;', '', 'g')
   return result
endfun

"" puDate to dc:date -------------------------------------------------
function! s:NewsCheckerParsePubDate(date)
   let date = a:date
   let month_Jan = '01'
   let month_Feb = '02'
   let month_Mar = '03'
   let month_Apr = '04'
   let month_May = '05'
   let month_Jun = '06'
   let month_Jul = '07'
   let month_Aug = '08'
   let month_Sep = '09'
   let month_Oct = '10'
   let month_Nov = '11'
   let month_Dec = '12'
   let day  = substitute(date, '\(..\) .*', '\1', '')
   let mon  = substitute(date, '.. \(...\) .... .*', '\1', '')
   let year = substitute(date, '.. ... \(....\) .*', '\1', '')
   let time = substitute(date, '.. ... .... \(.*\)', '\1', '')
   let date = year.'-'.month_{mon}.'-'.day.'T'.time
   return date
endfun

"" Initialise --------------------------------------------------------
function! s:NewsCheckerInit()
   if !exists('g:newschecker_comp_orememo')
      let g:newschecker_comp_orememo = 0
   endif
   if !exists('g:newschecker_cachedir')
      let g:newschecker_cachedir = '~/.newschecker/'
   endif
   if !exists('g:newschecker_file')
      let g:newschecker_file = g:newschecker_cachedir.'news.newschecker'
   endif
   if !exists('g:newschecker_ext')
      let g:newschecker_ext = '.newschecker'
   endif
   if !exists('g:newschecker_textwidth')
      let g:newschecker_textwidth = 80
   endif
   if g:newschecker_comp_orememo == 1
      let g:newschecker_ext      = '.orememo'
   endif
   if !exists('g:newschecker_w3mprg')
      let g:newschecker_w3mprg = ''
   endif
   if !exists('g:newschecker_curlprg')
      let g:newschecker_curlprg = 'curl'
   endif
   if !exists('g:newschecker_wwwprg')
      let g:newschecker_wwwprg = g:newschecker_w3mprg.' %URL%'
   endif

   let g:newschecker_cachedir = expand(g:newschecker_cachedir)
   let g:newschecker_file     = expand(g:newschecker_file)

   let f = 0
   while exists('g:newschecker_num'.f.'_url')
      let f = f + 1
      let s:all_num  = f
   endwhile

   if f == 0
      return
   endif

   command! NewsChecker call s:NewsChecker()
endfun

call s:NewsCheckerInit()

